use std::{time::{Duration, Instant}, sync::{Arc, RwLock}};

use crate::discord_invite::{DiscordInvite, DiscordInviteError};
use console::Term;
use getset::{Getters, MutGetters, Setters};
use rand::seq::IteratorRandom;
use reqwest::blocking::get;

#[derive(Clone, Debug, Getters, Setters)]
pub struct Bruter {
    #[getset(get = "pub")]
    found_codes: Vec<DiscordInvite>,
    #[getset(get = "pub", set = "pub")]
    codes: Vec<String>,
    iterations: usize,
}

impl Bruter {
    pub fn new(iterations: usize) -> Self {
        Self {
            found_codes: vec![],
            codes: vec![],
            iterations,
        }
    }

    pub fn start(mut self) -> anyhow::Result<()> {
        let mut rng = rand::thread_rng();
        for i in 0..self.iterations {
            let string: String = random_string::generate(
                (5..12).choose(&mut rng).unwrap(),
                random_string::charsets::ALPHANUMERIC,
            );
            self.codes.push(string);
        }

        for code in self.codes.clone() {
            let start = Instant::now();
            let res = DiscordInvite::resolve(code.clone());
            match res {
                Err(e) => {
                    println!("{e} | took {}ms", start.elapsed().as_millis());
                    match e {
                        DiscordInviteError::RateLimit(t) => {
                            std::thread::sleep(t + Duration::from_secs(1))
                        }
                        _ => (),
                    }
                }
                Ok(inv) => {
                    println!(
                        "Valid invite found for {}: https://discord.gg/{} | took {}ms",
                        &inv.guild.clone().unwrap().name.unwrap(),
                        &inv.code,
                        start.elapsed().as_millis()
                    );
                    self.found_codes.push(inv);
                }
            }
        }

        Ok(())
    }
}
